#ifndef _LTL_PACKED_ARRAY_H_
#define _LTL_PACKED_ARRAY_H_

/*
 * Packed arrays allow to efficiently map from LITTLE to C arrays.
 * Packed arrays are simply wrappers around C arrays, containing
 * runtime type information to support LITTLE's dynamic type system.
 */

#include <little/little.h>

#ifdef __cplusplus
extern "C" {
#endif

struct ltl_packed_array;
typedef struct ltl_packed_array ltl_packed_array_t;

/*
 * Creates a packed array.
 */
ltl_packed_array_t *
	ltl_packed_array_create(
		ltl_sym_t,		/* The cell type symbol.	*/
		size_t,			/* Number of cells in the array.*/
		size_t			/* Size of a single cell.	*/
	);

/*
 * Access a cell within the array.
 * Index check is performed.
 * You can access the underlying C array by using the following code (for
 * an example array of ints):
 * 	int *numbers = (int *) ltl_packed_array_at(packed_int_array,0);
 *	int n = numbers[0];
 */
void *					/* A pointer to the cell at index,
					   or NULL, if index is invalid.*/
	ltl_packed_array_at(
		ltl_packed_array_t *,	/* The array.			*/
		unsigned int		/* The cell index.		*/ 
	);

int
	ltl_packed_array_put(
		ltl_packed_array_t *,
		unsigned int,
		void *
	);

/*
 * Returns the size of this array.
 */
ltl_sym_t
	ltl_packed_array_size(
		ltl_packed_array_t *
	);

/*
 * Returns the LITTLE type ID of this array.
 */
ltl_sym_t
	ltl_packed_array_type(
		ltl_packed_array_t *
	);

/*
 * Disposes an array.
 */
void
	ltl_packed_array_dispose(
		ltl_packed_array_t *
	);

#ifdef __cplusplus
}
#endif

#endif
